package main

import (
	"os"
	"path/filepath"

	"go.wit.com/log"
)

func ReCheckReady() bool {
	return false
}

/*
-               // allrepos map[string]*repo
-
-               keys := make([]string, 0, len(me.allrepos))
-               for key := range me.allrepos {
-                       keys = append(keys, key)
-               }
-               sort.Strings(keys)
-
-               for _, path := range keys {
-                       repo := me.allrepos[path]
*/

func CheckReady() bool {
	if release.current == nil {
		log.Info("find the next repo first")
		return false
	}
	goSumS := release.current.GoState()
	dirtyS := release.current.State()
	lastS := release.current.Status.GetLastTagVersion()
	currentS := release.current.Status.GetCurrentBranchVersion()

	var targetS string
	targetS = release.version.String()

	log.Info("repo:", release.current.State(), goSumS, dirtyS, lastS, currentS, targetS)

	if goSumS == "RELEASED" {
		return true
	}
	if goSumS == "PRIMATIVE" {
		if targetS == lastS {
			release.current.SetGoState("RELEASED")
		}
		return true
	}
	if goSumS == "UNRELEASED" {
		return true
	}
	if goSumS == "READY" {
		if targetS == lastS {
			release.current.SetGoState("RELEASED")
			return true
		}
		if lastS == currentS {
			release.current.SetGoState("UNRELEASED")
		}
		return true
	}
	release.current.SetGoState("NOT READY")
	if release.current.Status.ReadOnly() {
		log.Info("\trepo is read only")
		return false
	}

	if targetS == lastS {
		log.Info("\trepo is already done", lastS, "=", targetS)
		release.current.SetGoState("READY")
		return true
	}
	if lastS == currentS {
		log.Info("\trepo is already done", lastS, "=", targetS)
		release.current.SetGoState("READY")
		return true
	}

	if goSumS == "BAD" {
		log.Info("\trepo is ready", release.current.State(), "BAD == BAD")
	} else {
		log.Info("\trepo is ready maybe", release.current.State(), "BAD !=", goSumS)
	}

	if release.current.Status.CheckDirty() {
		log.Info("\trepo is dirty")
		return false
	} else {
		log.Info("\trepo is ready", release.current.State(), "not dirty")
	}

	fullpath := filepath.Join(me.goSrcPwd.String(), release.current.State())

	testf := filepath.Join(fullpath, "go.mod")
	if Exists(testf) {
		log.Info("\trepo is not ready. go.mod exists")
		return false
	}

	testf = filepath.Join(fullpath, "go.sum")
	if Exists(testf) {
		log.Info("\trepo is not ready. go.sum exists")
		return false
	}

	testf = filepath.Join(fullpath, "LICENSE")
	if !Exists(testf) {
		log.Info("\trepo is not ready. missing LICENSE")
		return false
	}

	// final checks here
	if dirtyS == "unchanged" {
		log.Info("\trepo is ready", release.current.Name(), "unchanged")
	} else {
		log.Info("\trepo is not ready", dirtyS, "!= 'unchanged'")
		return false
	}

	curName := release.current.Status.GetCurrentBranchName()
	mName := release.current.Status.GetMasterBranchName()

	if curName == mName {
		log.Info("\trepo is ready working from main branch", curName, "=", mName)
	} else {
		log.Info("\trepo is not ready main branch", curName, "!=", mName)
		return false
	}

	release.current.SetGoState("READY")
	return true
}

// returns true if the file exists
func Exists(file string) bool {
	_, err := os.Stat(file)
	if err != nil {
		return false
	}
	return true
}
